@@ -15,8 +15,8 @@ require Exporter;
 
 @EXPORT_OK = qw(check_email
                 last_check
-	        check_hostname
-	        check_username);
+                check_hostname
+                check_username);
 $EXPORT_TAGS{constants} = [qw(CU_OK
                               CU_BAD_SYNTAX
                               CU_UNKNOWN_DOMAIN
@@ -26,14 +26,15 @@ $EXPORT_TAGS{constants} = [qw(CU_OK
                               CU_SMTP_UNREACHABLE)];
 push @EXPORT_OK, @{$EXPORT_TAGS{constants}};
 
-$VERSION = '1.15';
+$VERSION = '1.16';
 
 use Carp;
 use Net::DNS;
 use Net::SMTP;
 use IO::Handle;
 
-use vars qw($Skip_Network_Checks $Skip_SMTP_Checks $Skip_SYN
+use vars qw($Skip_Network_Checks $Skip_SMTP_Checks
+            $Skip_SYN $Net_DNS_Resolver
             $Timeout $Treat_Timeout_As_Fail $Debug
             $Sender_Addr $Helo_Domain $Last_Check);
 
@@ -54,15 +55,11 @@ $Sender_Addr = 'check@user.com';
 # sender domain used in HELO SMTP command - if undef lets
 # Net::SMTP use its default value
 $Helo_Domain = undef;
+# Default Net::DNS::Resolver override object
+$Net_DNS_Resolver = undef;
 # if true then enable debug mode
 $Debug = 0;
 
-# second half of ASCII table
-my $_SECOND_ASCII = '';
-for (my $i = 128; $i < 256; $i ++) {
-    $_SECOND_ASCII .= chr($i);
-}
-
 # check_email EMAIL
 sub check_email($);
 # last_check
@@ -95,7 +92,7 @@ sub check_email($) {
     my($email) = @_;
 
     unless(defined $email) {
-	croak __PACKAGE__ . "::check_email: \$email is undefined";
+        croak __PACKAGE__ . "::check_email: \$email is undefined";
     }
 
     _pm_log '=' x 40;
@@ -105,16 +102,16 @@ sub check_email($) {
     my($username, $hostname) = $email =~ /^(.*)@(.*)$/;
     # return false if it impossible
     unless(defined $hostname) {
-	return _result(CU_BAD_SYNTAX, 'bad address format: missing @');
+        return _result(CU_BAD_SYNTAX, 'bad address format: missing @');
     }
 
     my $ok = 1;
     $ok &&= check_hostname_syntax $hostname;
-    $ok &&= check_username_syntax $username if $ok;
+    $ok &&= check_username_syntax $username;
     if($Skip_Network_Checks) {
-	_pm_log "check_email: skipping network checks";
+        _pm_log "check_email: skipping network checks";
     } elsif ($ok) {
-	$ok &&= check_network $hostname, $username;
+        $ok &&= check_network $hostname, $username;
     }
 
     return $ok;
@@ -124,36 +121,43 @@ sub last_check() {
     return $Mail::CheckUser::Last_Check;
 }
 
+# build hostname regexp
+# NOTE: it doesn't strictly follow RFC822
+# because of what registrars now allow.
+my $DOMAIN_RE   = qr/(?:[\da-zA-Z]+ -+)* [\da-zA-Z]+/x;
+my $HOSTNAME_RE = qr/(?:$DOMAIN_RE \.)+ [a-zA-Z]+/x;
+
 sub check_hostname_syntax($) {
     my($hostname) = @_;
 
     _pm_log "check_hostname_syntax: checking \"$hostname\"";
 
     # check if hostname syntax is correct
-    # NOTE: it doesn't strictly follow RFC822
-    my $rAN = '[0-9a-zA-Z]';	# latin alphanum (don't use here \w: it can contain non-latin letters)
-    my $rDM = "(?:$rAN+-)*$rAN+"; # domain regexp
-    my $rHN = "(?:$rDM\\.)+$rDM"; # hostname regexp
-    if($hostname =~ /^$rHN$/o) {
-	return _result(CU_OK, 'correct hostname syntax');
+    if($hostname =~ /^ $HOSTNAME_RE $/xo) {
+        return _result(CU_OK, 'correct hostname syntax');
     } else {
-	return _result(CU_BAD_SYNTAX, 'bad hostname syntax');
+        return _result(CU_BAD_SYNTAX, 'bad hostname syntax');
     }
 }
 
+# build username regexp
+# NOTE: it doesn't strictly follow RFC821
+my $STRING_RE = ('[' . quotemeta(join '',
+                                 grep(!/[<>()\[\]\\\.,;:\@"]/, # ["], UnBug Emacs
+                                      map chr, 33 .. 126)) . ']');
+my $USERNAME_RE = qr/(?:$STRING_RE+ \.)* $STRING_RE+/x;
+
+
 sub check_username_syntax($) {
     my($username) = @_;
 
     _pm_log "check_username_syntax: checking \"$username\"";
 
     # check if username syntax is correct
-    # NOTE: it doesn't strictly follow RFC821
-    my $rST = '[^ <>\(\)\[\]\\\.,;:@"' . $_SECOND_ASCII . ']'; # allowed string regexp
-    my $rUN = "(?:$rST+\\.)*$rST+"; # username regexp
-    if($username =~ /^$rUN$/o) {
-	return _result(CU_OK, 'correct username syntax');
+    if($username =~ /^ $USERNAME_RE $/xo) {
+        return _result(CU_OK, 'correct username syntax');
     } else {
-	return _result(CU_BAD_SYNTAX, 'bad username syntax');
+        return _result(CU_BAD_SYNTAX, 'bad username syntax');
     }
 }
 
@@ -168,10 +172,10 @@ sub check_network($$) {
     my $timeout = $Timeout;
     my $start_time = time;
 
-    my $resolver = new Net::DNS::Resolver;
+    my $resolver = $Mail::CheckUser::Net_DNS_Resolver || new Net::DNS::Resolver;
     my $tout = _calc_timeout($timeout, $start_time);
     return _result(CU_DNS_TIMEOUT, 'DNS timeout') if $tout == 0;
-    $resolver->tcp_timeout($tout);
+    $resolver->udp_timeout($tout);
 
     my @mx = mx($resolver, "$hostname.");
     $tout = _calc_timeout($timeout, $start_time);
@@ -179,35 +183,30 @@ sub check_network($$) {
 
     # check result of query
     if(@mx) {
-	# if MX record exists ...
-
-	my %mservers = ();
-	foreach my $rr (@mx) {
-	    $mservers{$rr->exchange} = $rr->preference;
-	}
-	# here we get list of mail servers sorted by preference
-	@mservers = sort { $mservers{$a} <=> $mservers{$b} } keys %mservers;
+        # if MX record exists,
+        # then it's already sorted by preference
+        @mservers = map {$_->exchange} @mx;
     } else {
-	# if there is no MX record try hostname as mail server
-	my $tout = _calc_timeout($timeout, $start_time);
-	return _result(CU_DNS_TIMEOUT, 'DNS timeout') if $tout == 0;
-	$resolver->tcp_timeout($tout);
+        # if there is no MX record try hostname as mail server
+        my $tout = _calc_timeout($timeout, $start_time);
+        return _result(CU_DNS_TIMEOUT, 'DNS timeout') if $tout == 0;
+        $resolver->udp_timeout($tout);
 
-	my $res = $resolver->search("$hostname.", 'A');
-	# check if timeout has happen
-	$tout = _calc_timeout($timeout, $start_time);
-	return _result(CU_DNS_TIMEOUT, 'DNS timeout') if $tout == 0;
+        my $res = $resolver->search("$hostname.", 'A');
+        # check if timeout has happen
+        $tout = _calc_timeout($timeout, $start_time);
+        return _result(CU_DNS_TIMEOUT, 'DNS timeout') if $tout == 0;
 
-	# check result of query
-	if($res) {
-	    @mservers = ($hostname);
-	} else {
-	    return _result(CU_UNKNOWN_DOMAIN, 'DNS failure: ' . $resolver->errorstring);
-	}
+        # check result of query
+        if($res) {
+            @mservers = ($hostname);
+        } else {
+            return _result(CU_UNKNOWN_DOMAIN, 'DNS failure: ' . $resolver->errorstring);
+        }
     }
 
     if($Skip_SMTP_Checks) {
-	return _result(CU_OK, 'skipping SMTP checks');
+        return _result(CU_OK, 'skipping SMTP checks');
     } else {
         if ($Skip_SYN) {
             # Skip SYN/ACK check.
@@ -286,8 +285,8 @@ sub check_user_on_host($$$$) {
     my @hello_params = defined $Helo_Domain ? (Hello => $Helo_Domain) : ();
     my $smtp = Net::SMTP->new($mserver, Timeout => $tout, @hello_params);
     unless(defined $smtp) {
-	_pm_log "check_user_on_host: unable to connect to \"$mserver\"";
-	return -1;
+        _pm_log "check_user_on_host: unable to connect to \"$mserver\"";
+        return -1;
     }
 
     # try to check if user is valid with MAIL/RCPT commands
@@ -297,30 +296,30 @@ sub check_user_on_host($$$$) {
 
     # send MAIL FROM command
     unless($smtp->mail($Sender_Addr)) {
-	# something wrong?
+        # something wrong?
 
-	# check for timeout
-	return _result(CU_SMTP_TIMEOUT, 'SMTP timeout') if $tout == 0;
+        # check for timeout
+        return _result(CU_SMTP_TIMEOUT, 'SMTP timeout') if $tout == 0;
 
-	_pm_log "check_user_on_host: can't say MAIL - " . $smtp->message;
-	return -1;
+        _pm_log "check_user_on_host: can't say MAIL - " . $smtp->message;
+        return -1;
     }
 
     # send RCPT TO command
     if($smtp->to("$username\@$hostname")) {
-	return _result(CU_OK, 'SMTP server accepts username');
+        return _result(CU_OK, 'SMTP server accepts username');
     } else {
-	# check if verify returned error because of timeout
-	my $tout = _calc_timeout($timeout, $start_time);
-	return _result(CU_SMTP_TIMEOUT, 'SMTP timeout') if $tout == 0;
+        # check if verify returned error because of timeout
+        my $tout = _calc_timeout($timeout, $start_time);
+        return _result(CU_SMTP_TIMEOUT, 'SMTP timeout') if $tout == 0;
 
-	if($smtp->code == 550 or $smtp->code == 551 or $smtp->code == 553) {
-	    return _result(CU_UNKNOWN_USER, 'no such user');
-	} else {
-	    return _result(CU_OK, 'unknown error in response');
-	    _pm_log "check_user_on_host: unknown error in response";
-	    return 1;
-	}
+        if($smtp->code == 550 or $smtp->code == 551 or $smtp->code == 553) {
+            return _result(CU_UNKNOWN_USER, 'no such user');
+        } else {
+            return _result(CU_OK, 'unknown error in response');
+            _pm_log "check_user_on_host: unknown error in response";
+            return 1;
+        }
     }
 
 
@@ -339,9 +338,9 @@ sub _calc_timeout($$) {
     my $timeout = $full_timeout - $passed_time;
 
     if($timeout < 0) {
-	return 0;
+        return 0;
     } else {
-	return $timeout;
+        return $timeout;
     }
 }
 
@@ -349,7 +348,7 @@ sub _pm_log($) {
     my($log_str) = @_;
 
     if($Debug) {
-	print STDERR "$log_str\n";
+        print STDERR "$log_str\n";
     }
 }
 
@@ -364,8 +363,8 @@ sub _result($$) {
     $ok = 1 if $code == CU_SMTP_TIMEOUT and not $Treat_Timeout_As_Fail;
 
     $Last_Check = { ok     => $ok,
-		    code   => $code,
-		    reason => $reason };
+                    code   => $code,
+                    reason => $reason };
 
     my($sub) = (caller(1))[3] =~ /^.*::(.*)$/;
 
@@ -475,7 +474,7 @@ valid.
     if(check_email($email)) {
         print "E-mail address <$email> is OK\n";
     } else {
-	print "E-mail address <$email> isn't valid: ",
+        print "E-mail address <$email> isn't valid: ",
               last_check()->{reason}, "\n";
     }
 
@@ -575,9 +574,9 @@ checks are performed.  By default it is false.
 
 =item $Mail::CheckUser::Skip_SYN
 
-If it is true module uses L<Net::Ping|Net::Ping> to determine remote
-reachability of SMTP servers before doing SMTP checks.  By default it
-is true.
+By default L<Net::Ping|Net::Ping> is used to determine remote
+reachability of SMTP servers before doing SMTP checks.
+Setting this to true skips this check.  By default it is false.
 
 =item $Mail::CheckUser::Sender_Addr
 
@@ -599,6 +598,14 @@ Timeout in seconds for network checks.  By default it is C<60>.
 If it is true C<Mail::CheckUser> treats checks that time out as
 failed.  By default it is false.
 
+=item $Mail::CheckUser::Net_DNS_Resolver
+
+Override with customized Net::DNS::Resolver object.
+This is used to lookup MX and A records for the
+email domain when network checks are enabled.
+If undef, Net::DNS::Resolver->new will be used.
+The default value is undef.
+
 =item $Mail::CheckUser::Debug
 
 If it is true then enable debug output on C<STDERR>.  By default it is
